;**********************	Software Single Master I2C Driver ***************

cycle_4_7us	equ cycle_1ms / 213 + 1
cycle_4us	equ cycle_1ms / 250 + 1
$noli

mi2cdelay	macro delay
	  if (delay < 32768)		;;skip negative value
	    rept delay
	      nop
	    endm
	  endif
	endm
$li
;**********************	basic I2C routines ******************************
;----------------------	send start / repeat start -----------------------
;SDA: 1 -> 0,	bzw. X -> 0
;SCL: 1 -> 0,	bzw. 0 -> 0
SI2CSTA:
	setb ssda
	setb sscl
	mi2cdelay cycle_4_7us - 1
	clr ssda
	mi2cdelay cycle_4us - 1
	clr sscl
	ret
;----------------------	send stop ---------------------------------------
;Input:	CY = 0: no previous error
;Output: CY = CY OR 1: Error, SDA or SCL still low
;SDA: X -> 1
;SCL: 0 -> 1
SI2CSTO:
	clr ssda
	mi2cdelay cycle_4us - 1
	setb sscl
	mi2cdelay cycle_4_7us - 3
	orl c, /sscl		;test sscl
	setb ssda
	nop			;time to go high
	orl c, /ssda		;test ssda
	ret
;----------------------	reset the I2C bus -------------------------------
;Output: CY = 1 Error, reset failed
;SDA: X -> 1
;SCL: X -> 1
SI2CRES:
	mov r2, #9		;try it 9 times
?si2crs1:
	clr c
	clr sscl
	call si2csto		;try sending stop
	jnc ?si2crs2
	djnz r2, ?si2crs1
?si2crs2:
	ret
;----------------------	Send Byte, Receive ACK --------------------------
;Input: A = Byte
;Output: CY = 1: no ACK received
;Used: R2
;SDA: X -> 1
;SCL: 0 -> 0
SI2COUT:
	setb c
	sjmp ?si2cio1
;----------------------	Receive Byte, Send ACK or NACK ------------------
;Input: CY = 0: Send ACK, 1: Send NACK
;Output: A = Byte
;Used: R2
SI2CIN:	mov a, #0ffh
?si2cio1:
	mov r2, #9		;8 Bit + ACK
?si2cio2:
	rlc a			;MSB first
	mov ssda, c		;bit out
	mi2cdelay cycle_4_7us - 6
	setb sscl
	mi2cdelay cycle_4us - 2
	mov c, ssda		;bit in
	clr sscl
	djnz r2, ?si2cio2
	ret
;------------------------------------------------------------------------
