;************************************************************************
;*									*
;*		Read / Write EEPROM 24C02 ... 24C256			*
;*									*
;************************************************************************

Pagesize	equ 3		;2^3 = 8 Byte per page

;----------------------	Subroutine: set EEPROM address ------------------
;Input: R7, R6 = Address (11 Bit)
;	CY = 0: start + high, low address for read or write data
;	CY = 1: repeat start + high address for read data
;Address byte: 1   0   1   0 a10  a9  a8  rd	(x24c16)
;	       1   0   1   0   0   0   0  rd 	(x24c64)
SEEPAD:	call si2csta		;start
if(ExtAdrMode)
	mov a, #0A0h / 2	;I2C address
	rlc a			;insert read bit
	jb acc.0, ?sead1
	call si2cout
	mov a, r7		;high address
else
	mov a, r7
	anl a, #7		;mask 2kB
	rlc a
	orl a, #0a0h
	jb acc.0, ?sead1
endif
	call si2cout		;high address
	mov a, r6
?sead1:	jmp si2cout		;low address




;**********************	Serial EEPROM: Block Read ***********************
;Input:	R7, R6 = Address (11Bit)
;	R5 = byte number (max 256 byte)
;	R0 = point to 1. byte
;Used:	A, R2
SEEPRD:	clr c
	call seepad		;send address
?serd1:	setb c
	call seepad		;repeat start, set read mode
	jc ?serd5		;error, NACK received
?serd2:	inc r6			;increment address
	cjne r6, #0, ?serd3
	inc r7			;next page
if(ExtAdrMode)
?serd3:	djnz r5, ?serd4
	setb c			;last byte to read, send NACK
?serd4:	call si2cin
	mov @r0, a
	inc r0
	cjne r5, #0, ?serd2	;address next page
else
	setb c			;last byte of page, send NACK
?serd3:	djnz r5, ?serd4
	setb c			;last byte to read, send NACK
?serd4:	mov ac, c		;save CY
	call si2cin
	mov @r0, a
	inc r0
	jnb ac, ?serd2		;read next byte
	cjne r5, #0, ?serd1	;address next page
endif
?serd5:	jmp si2csto		;send stop

;**********************	Serial EEPROM: Block Write **********************
;Input:	R7, R6 = Address (11Bit)
;	R5 = byte number (max 256 byte)
;	R0 = point to 1. byte
;Used:	A, B, R2

?sewr1:	mov b, r6		;to test page limit
	clr b.pagesize+1
?sewr2:	jb b.pagesize+1, ?sewr3
	inc b
	mov a, @r0
	call si2cout
	jc ?sewr6		;Error, NACK received
	inc r0
	inc r6
	djnz r5, ?sewr2		;next byte
?sewr3:	call si2csto
	cjne r6, #0, SEEPWR
	inc r7

SEEPWR:	mov b, #0
?sewr4:	clr c
	call seepad		;send address
	jnc ?sewr5		;ACK polling until write done
	call si2csto
	djnz b, ?sewr4
	ret			;Timeout
?sewr5:	cjne r5, #0, ?sewr1	;last byte written ?
?sewr6:	jmp si2csto
;-----------------------------------------------------------------------
